#
# Script for checking if MIG7 v1.6 files exist
#
# Usage: xtclsh check_ddr3_if_mig7_v1_6.tcl <model>
# eg.
#        xtclsh check_ddr3_if_mig7_v1_6.tcl admxrc6t1
#
set mig_ver 1_6
#
proc show_usage { } {
  puts {Usage:}
  puts {}
  puts {  xtclsh check_ddr3_if_mig7_v${mig_ver}.tcl -? | -h | --help}
  puts {  xtclsh check_ddr3_if_mig7_v${mig_ver}.tcl <model>}
  puts {}
  puts {-? -h --help      Show this message.}
}
#
for { set i 0 } { $i < $argc } { incr i } {
  set argvi [ lindex $argv $i ];
  if { [ string compare ${argvi} {-?} ] == 0 || [ string compare ${argvi} {-h} ] == 0 || [ string compare ${argvi} {--help} ] == 0 } {
    show_usage
    exit 0
  }
  if { [ string index ${argvi} 0 ] == {-} } {
    puts "*** Unrecognised option '${argvi}'"
    show_usage
    exit -1
  } else {
    break
  }
}
if { $i < $argc } {
  set _Model [ lindex $argv $i ]
} else {
  puts {*** Missing model. Please specify a model, e.g. admxrc7k1.}
  show_usage
  exit -1
}
#
# Ensure that the current working directory is that of this script
set _script_path [ file dirname [ info script ] ]
cd ${_script_path}
#
set _PRJ_Path ./${_Model}/mig7_v${mig_ver}/${_Model}.prj
#
#
if {[file exists ${_PRJ_Path}]} {
  puts ""
  puts "Model ${_Model} selected."
} else {
  puts ""
  puts "Missing files for model '${_Model}':"
  puts ""
  puts "PRJ file: ${_PRJ_Path}"
  puts ""
  puts "Please specify a valid model, e.g. admxrc7k1."
  puts ""
  exit -1
}
#
#
# Get version number from project file
set prj_file [open "${_PRJ_Path}" r]
set file_data [read $prj_file]
close $prj_file

puts ""
puts "Searching ${_PRJ_Path} for MIG7 version..."
puts ""
set found 0
set data [split $file_data "\n"]
foreach line $data {
  if {[regexp -nocase "<version>(.*)</version>" ${line} result ver_num]} {
    incr found
#    puts "  ($found): ${result}, ${ver_num}"
    set _MIG_sub [regsub -all {\.} ${ver_num} "_"]
  }
}

if {${_MIG_sub} ne ${mig_ver}} {
  puts "Error: MIG7 version in ${_PRJ_Path} (${_MIG_sub}) is different from ${mig_ver}"
  exit -1
}

if {${found} eq 1} {
  set _MIG_Version v[regsub {\.} ${ver_num} "_"]
  puts "Selecting MIG7 version ${ver_num} (${_MIG_Version})"
} else {
  puts "Error: Found more than 1 ${found} version entries in ${_PRJ_Path}"
  exit -1
}
#
#
set _RTL_Path ./${_Model}/mig7_v${mig_ver}/rtl
set _Error 0
#
#
proc test_file {_File} {
  global _Error
  if {![file exists $_File]} {
    puts "$_File missing"
    incr _Error
  } else {
    puts "$_File found"
  }
}
#
#
puts ""
puts "Checking MIG7 ${_MIG_Version} files for ${_Model}:"
#
test_file ${_RTL_Path}/ui/ui_cmd.v
test_file ${_RTL_Path}/ui/ui_rd_data.v
test_file ${_RTL_Path}/ui/ui_wr_data.v
test_file ${_RTL_Path}/ui/ui_top.v
#
test_file ${_RTL_Path}/phy/ddr_byte_group_io.v
test_file ${_RTL_Path}/phy/ddr_byte_lane.v
test_file ${_RTL_Path}/phy/ddr_calib_top.v
test_file ${_RTL_Path}/phy/ddr_if_post_fifo.v
test_file ${_RTL_Path}/phy/ddr_mc_phy.v
test_file ${_RTL_Path}/phy/ddr_mc_phy_wrapper.v
test_file ${_RTL_Path}/phy/ddr_of_pre_fifo.v
test_file ${_RTL_Path}/phy/ddr_phy_4lanes.v
test_file ${_RTL_Path}/phy/ddr_phy_ck_addr_cmd_delay.v
test_file ${_RTL_Path}/phy/ddr_phy_dqs_found_cal.v
test_file ${_RTL_Path}/phy/ddr_phy_init.v
test_file ${_RTL_Path}/phy/ddr_phy_rdlvl.v
test_file ${_RTL_Path}/phy/ddr_phy_wrcal.v
test_file ${_RTL_Path}/phy/ddr_phy_wrlvl.v
test_file ${_RTL_Path}/phy/ddr_prbs_gen.v
test_file ${_RTL_Path}/phy/ddr_phy_top.v
test_file ${_RTL_Path}/phy/ddr_phy_oclkdelay_cal.v
#
test_file ${_RTL_Path}/controller/arb_mux.v
test_file ${_RTL_Path}/controller/arb_row_col.v
test_file ${_RTL_Path}/controller/arb_select.v
test_file ${_RTL_Path}/controller/bank_cntrl.v
test_file ${_RTL_Path}/controller/bank_common.v
test_file ${_RTL_Path}/controller/bank_compare.v
test_file ${_RTL_Path}/controller/bank_mach.v
test_file ${_RTL_Path}/controller/bank_queue.v
test_file ${_RTL_Path}/controller/bank_state.v
test_file ${_RTL_Path}/controller/col_mach.v
test_file ${_RTL_Path}/controller/rank_cntrl.v
test_file ${_RTL_Path}/controller/rank_common.v
test_file ${_RTL_Path}/controller/rank_mach.v
test_file ${_RTL_Path}/controller/round_robin_arb.v
test_file ${_RTL_Path}/controller/mc.v
#
test_file ${_RTL_Path}/clocking/infrastructure.v
#
test_file ${_RTL_Path}/ip_top/mem_intfc.v
test_file ${_RTL_Path}/ip_top/memc_ui_top_std.v
#
if {${_Error} eq 0} {
  puts ""
  puts "CORE checker completed sucessfully."
  puts ""
  exit 0
} else {
  puts ""
  puts "CORE checker complete with ${_Error} errors detected."
  puts ""
  puts "All DDR3 MIG7 ${_MIG_Version} files for ${_Model} have not been found."
  puts ""
  puts "Please re-run this macro file after generating the"
  puts "DDR3 MIG7 ${_MIG_Version} files by running"
  puts ""
  puts "  xtclsh gen_ddr3_if_mig7_v${ver_num}.tcl ${_Model}"
  puts ""
  puts "in a command console in directory:"
  puts ""
  puts "  hdl/vhdl/common/ddr3_sdram_if"
  puts ""
  exit -1
}