--==============================================================================
--
--     File Name: ddr3_if_bank_lwb_s7.vhd
--       Version: 0.1
--          Date: 26 Nov 2012
--         Model: synth
--
--  Dependencies: Light weight behavioural DDR3 SDRAM bank interface
--
--       Company: Alpha Data
--        Design: Light weight behavioural DDR3 SDRAM bank interface
--
--   Description: OCP to Light weight behavioural DDR3 SDRAM bank interface
--
--   Limitations: 7 series models
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2012 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 26 Nov 2012: Initial design
--
--==============================================================================


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_misc.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;

library work;
use work.adb3_ocp.all;
use work.adb3_ocp_comp.all;
use work.adb3_target_inc_pkg.all;
use work.ddr3_if_pkg.all;

entity ddr3_if_bank is
generic(
  sim   : boolean := false;
  clock : mig_clocks_t := MIG_CLOCKS_800;
  bank  : integer range 0 to DDR3_BANKS-1 := 0);
port(
  -- OCP interface
  ocp_rst               : in    std_logic;
  ocp_clk               : in    std_logic;
  ocp_m2s               : in    adb3_ocp_m2sT;
  ocp_s2m               : out   adb3_ocp_s2mT;
  -- Memory interface
  ddr3_rst              : in    std_logic;
  ddr3_ref_clk          : in    std_logic;
  ddr3_clk              : in    std_logic;
  ddr3_iodelay_ctrl_rdy : in    std_logic;
  -- Memory status
  ddr3_if_rdy           : out   std_logic;
  ddr3_if_stat          : out   std_logic_vector(3 downto 0);
  ddr3_if_err           : out   std_logic_vector(3 downto 0);
  -- Physical memory interface
  ddr3_addr_out         : out   ddr3_addr_out_t;
  ddr3_ctrl_out         : out   ddr3_ctrl_out_t;
  ddr3_data_inout       : inout ddr3_data_inout_t;
  ddr3_clk_out          : out   ddr3_clk_out_t;
  -- Debug info
  ddr3_if_debug         : out   std_logic_vector(31 downto 0));
end;


architecture behavioural of ddr3_if_bank is

  component mig_ddr3_lwb
  generic(
    message_level  : natural range 0 to 7 := 0;
    app_addr_width : natural;
    app_data_width : natural;
    app_mask_width : natural);
  port(
    rst               : in    std_logic;
    clk               : in    std_logic;
    -- MIG user interface inputs
    app_addr          : in    std_logic_vector(app_addr_width-1 downto 0);
    app_cmd           : in    std_logic_vector(2 downto 0);
    app_en            : in    std_logic;
    app_sz            : in    std_logic;
    app_wdf_data      : in    std_logic_vector(app_data_width-1 downto 0);
    app_wdf_end       : in    std_logic;
    app_wdf_mask      : in    std_logic_vector(app_mask_width-1 downto 0);
    app_wdf_wren      : in    std_logic;
    -- MIG user interface outputs
    dfi_init_complete : out   std_logic;
    app_rd_data       : out   std_logic_vector(app_data_width-1 downto 0);
    app_rd_data_valid : out   std_logic;
    app_rdy           : out   std_logic;
    app_wdf_rdy       : out   std_logic;
    -- File control
    init_start        : in    boolean := false;
    init_filename     : in    string := "init.txt";
    log_start         : in    boolean := false;
    log_filename      : in    string := "log.txt");
  end component;


  -- Overridden by global setting in simulation .do file
  constant MESSAGE_LEVEL : natural := 0;

  -- Bank init ang log file names
  constant INIT_FILENAME : string  := "ddr3_lwb_init_bank" & integer'image(bank) & ".txt";
  constant LOG_FILENAME  : string  := "ddr3_lwb_log_bank"  & integer'image(bank) & ".txt";

  -- Select speed bin constants
  constant MIG_CLOCKS : mig_clocks_t := clock;

  -- MIG_CLOCKS, MIG_COMMON constants package ddr3_if_pkg
  constant APP_DATA_WIDTH : natural := 2 * MIG_CLOCKS.nCK_PER_CLK * MIG_COMMON.PAYLOAD_WIDTH;
  constant APP_MASK_WIDTH : natural := APP_DATA_WIDTH / 8;
  constant DATA_WORDS     : natural := APP_DATA_WIDTH / MIG_COMMON.DQ_WIDTH;

  signal mig_rst_vec       : std_logic_vector(31 downto 0) := (others => '1');
  signal mig_rst           : std_logic;
  signal mig_ddr3_if_clk   : std_logic;
  signal dfi_init_complete : std_logic;

  -- memc_ui_top signals
  signal app_sz            : std_logic;
  signal app_wdf_wren      : std_logic;
  signal app_wdf_data      : std_logic_vector(app_data_width-1 downto 0);
  signal app_wdf_mask      : std_logic_vector(app_mask_width-1 downto 0);
  signal app_wdf_end       : std_logic;
  signal app_addr          : std_logic_vector(MIG_COMMON.ADDR_WIDTH-1 downto 0);
  signal app_cmd           : std_logic_vector(2 downto 0);
  signal app_en            : std_logic;
  signal app_rdy           : std_logic;
  signal app_wdf_rdy       : std_logic;
  signal app_rd_data       : std_logic_vector(app_data_width-1 downto 0);
  signal app_rd_data_valid : std_logic;

  -- File control signals
  signal init_start : boolean := false;
  signal log_start  : boolean := false;

begin

  -- Check MIG interface address widths
  assert not(DDR3_BANK_ROW_WIDTH /= MIG_COMMON.ROW_WIDTH)
    report "(ddr3_if_bank): MIG ROW_WIDTH /= target_inc_pkg DDR3_BANK_ROW_WIDTH"
    severity failure;

  -- Should be Rank(1)+Bank(3)+Row(13/14)+Col(10) and remove Rank (-1) and convert to byte addressing (+2)
  assert not(DDR3_BYTE_ADDR_WIDTH /= (MIG_COMMON.ADDR_WIDTH - 1 + 2))
    report "(ddr3_if_bank): MIG ADDR_WIDTH /= target_inc_pkg DDR3_BYTE_ADDR_WIDTH"
    severity failure;

  assert not(DDR3_BANK_DATA_WIDTH /= MIG_COMMON.DQ_WIDTH)
    report "(ddr3_if_bank): MIG DQ_WIDTH /= target_inc_pkg DDR3_BANK_DATA_WIDTH"
    severity failure;


  -- Replace MIG infrastructure block with clock and reset generation
  -- Use ddr3_ref_clk as ddr3_clk can be 2 x ddr3_ref_clk
  mig_rst_p : process(ddr3_rst,ddr3_ref_clk)
  begin
    if (ddr3_rst = '1') then
      mig_rst_vec <= (others => '1');
    elsif rising_edge(ddr3_ref_clk) then
      mig_rst_vec <= std_logic_vector(unsigned(mig_rst_vec) srl 1);
    end if;
  end process;

  mig_rst <= mig_rst_vec(0);

  mig_clock_g : if (MIG_CLOCKS.tCK = 2500)or(MIG_CLOCKS.tCK = 1250) generate
    -- Select 200MHz reference clock for DDR-800 and DDR-1600
    mig_ddr3_if_clk <= ddr3_ref_clk;
  end generate;

  mig_clock_div_g : if (MIG_CLOCKS.tCK = 1875)or(MIG_CLOCKS.tCK = 1500) generate
  -- Select 200MHz reference clock divided by 2 for DDR-1066 and DDR-1333
    assert false
      report "(ddr3_if_bank): Selecting 100MHz mig_ddr3_if_clk for DDR-1066/DDR-1333"
      severity warning;

    mig_clk_p : process(ddr3_rst,ddr3_ref_clk)
    begin
      if (ddr3_rst = '1') then
        mig_ddr3_if_clk <= '0';
      elsif rising_edge(ddr3_ref_clk) then
        mig_ddr3_if_clk <= not mig_ddr3_if_clk;
      end if;
    end process;
  end generate;


  -- Instantiate OCP to MIG block
  adb3_ocp_ocp2ddr3_nb_i : adb3_ocp_ocp2ddr3_nb
  generic map(
    app_row_width     => MIG_COMMON.ROW_WIDTH,     -- natural
    app_col_width     => MIG_COMMON.COL_WIDTH,     -- natural
    app_bank_width    => MIG_COMMON.BANK_WIDTH,    -- natural
    app_addr_width    => MIG_COMMON.ADDR_WIDTH,    -- natural
    app_dqs_cnt_width => MIG_COMMON.DQS_CNT_WIDTH) -- natural := 2
  port map(
    -- OCP Interface
    ocp_rst           => ocp_rst,           -- in    std_logic
    ocp_clk           => ocp_clk,           -- in    std_logic
    ocp_m2s           => ocp_m2s,           -- in    adb3_ocp_m2sT
    ocp_s2m           => ocp_s2m,           -- out   adb3_ocp_s2mT
    -- MIG user interface
    mig_rst           => mig_rst,           -- in    std_logic
    mig_clk           => mig_ddr3_if_clk,   -- in    std_logic
    phy_init_done     => dfi_init_complete, -- in    std_logic
    app_rdy           => app_rdy,           -- in    std_logic
    app_wdf_rdy       => app_wdf_rdy,       -- in    std_logic
    app_rd_data       => app_rd_data,       -- in    adb3_ocp_data_s
    app_rd_data_valid => app_rd_data_valid, -- in    std_logic
    app_en            => app_en,            -- out   std_logic
    app_cmd           => app_cmd,           -- out   std_logic_vector(2 downto 0)
    app_addr          => app_addr,          -- out   std_logic_vector(app_addr_width-1 downto 0);
    app_sz            => app_sz,            -- out   std_logic
    app_wdf_wren      => app_wdf_wren,      -- out   std_logic
    app_wdf_data      => app_wdf_data,      -- out   adb3_ocp_data_s
    app_wdf_mask      => app_wdf_mask,      -- out   adb3_ocp_be_s
    app_wdf_end       => app_wdf_end,       -- out   std_logic
    -- Debug info
    debug_info        => ddr3_if_debug);    -- out   std_logic_vector(31 downto 0))

  -- Check app_sz against app_addr LSBs
  mig_chk_p : process(ddr3_ref_clk)
  begin
    if rising_edge(ddr3_ref_clk) then
      -- 2 cycle command (DATA_WORDS = 4) => app_addr is 8-location aligned
      assert not((app_en = '1')and(app_sz = '1')and(DATA_WORDS = 4)and(OR_reduce(app_addr(2 downto 0)) = '1'))
        report "(ddr3_if_bank): 2 cycle command (8 word) with app_addr(2:0) non-zero."
        severity error;

      -- 2 cycle command (DATA_WORDS = 8) => ILLEGAL
      assert not((app_en = '1')and(app_sz = '1')and(DATA_WORDS = 8))
        report "(ddr3_if_bank): 2 cycle command (16 word) is illegal."
        severity error;

      -- 1 cycle command (DATA_WORDS = 4) => app_addr is 4-location aligned
      assert not((app_en = '1')and(app_sz = '0')and(DATA_WORDS = 4)and(OR_reduce(app_addr(1 downto 0)) = '1'))
        report "(ddr3_if_bank): 1 cycle command (4 word) with app_addr(1:0) non-zero."
        severity error;

      -- 1 cycle command (DATA_WORDS = 8) => app_addr is 8-location aligned
      assert not((app_en = '1')and(app_sz = '0')and(DATA_WORDS = 8)and(OR_reduce(app_addr(2 downto 0)) = '1'))
        report "(ddr3_if_bank): 1 cycle command (8 word) with app_addr(2:0) non-zero."
        severity error;
    end if;
  end process;


  -- Use ddr3_data_inout as log/init inputs from testbench
  init_start <= true when (ddr3_data_inout.dqs_p(0) = '1') else false;
  log_start  <= true when (ddr3_data_inout.dqs_n(0) = '1') else false;

  -- Replace MIG core and DDR3 SDRAM model with light weight behavioural block
  mig_ddr3_lwb_i : mig_ddr3_lwb
  generic map(
    message_level => MESSAGE_LEVEL,
    app_addr_width => MIG_COMMON.ADDR_WIDTH, -- natural
    app_data_width => APP_DATA_WIDTH,        -- natural
    app_mask_width => APP_MASK_WIDTH)        -- natural
  port map(
    rst               => mig_rst,           -- in    std_logic
    clk               => mig_ddr3_if_clk,   -- in    std_logic
    -- MIG user interface inputs
    app_addr          => app_addr,          -- in    std_logic_vector(app_addr_width-1 downto 0)
    app_cmd           => app_cmd,           -- in    std_logic_vector(2 downto 0)
    app_en            => app_en,            -- in    std_logic
    app_sz            => app_sz,            -- in    std_logic
    app_wdf_data      => app_wdf_data,      -- in    std_logic_vector(app_data_width-1 downto 0)
    app_wdf_end       => app_wdf_end,       -- in    std_logic
    app_wdf_mask      => app_wdf_mask,      -- in    std_logic_vector(app_mask_width-1 downto 0)
    app_wdf_wren      => app_wdf_wren,      -- in    std_logic
    -- MIG user interface outputs
    dfi_init_complete => dfi_init_complete, -- out   std_logic
    app_rd_data       => app_rd_data,       -- out   std_logic_vector(app_data_width-1 downto 0)
    app_rd_data_valid => app_rd_data_valid, -- out   std_logic
    app_rdy           => app_rdy,           -- out   std_logic
    app_wdf_rdy       => app_wdf_rdy,       -- out   std_logic
    -- File control
    init_start        => init_start,        -- in    boolean := false;
    init_filename     => INIT_FILENAME,     -- in    string := "init.txt";
    log_start         => log_start,         -- in    boolean := false;
    log_filename      => LOG_FILENAME);     -- in    string := "log.txt");

  process(mig_ddr3_if_clk)
  begin
    if rising_edge(mig_ddr3_if_clk) then
      -- Connect ready
      ddr3_if_rdy <= dfi_init_complete;
    end if;
  end process;


  -- Tie-off physical memory interface
  ddr3_addr_out   <= DDR3_ADDR_DEFAULT_OUT;
  ddr3_ctrl_out   <= DDR3_CTRL_DEFAULT_OUT;
  ddr3_data_inout <= DDR3_DATA_DEFAULT_INOUT;
  ddr3_clk_out    <= DDR3_CLK_DEFAULT_OUT;

  -- Connect status
  ddr3_if_stat(0) <= dfi_init_complete;
  ddr3_if_stat(1) <= dfi_init_complete;
  ddr3_if_stat(2) <= dfi_init_complete;
  ddr3_if_stat(3) <= dfi_init_complete;

  -- Connect error
  ddr3_if_err(0)  <= mig_rst;
  ddr3_if_err(1)  <= mig_rst;
  ddr3_if_err(2)  <= mig_rst;
  ddr3_if_err(3)  <= mig_rst;

end;
